@@ -26,6 +26,7 @@ import (
 	"reflect"
 	"strconv"
 	"strings"
+	"syscall"
 
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/printer"
@@ -60,35 +61,38 @@ const (
 	clIdentCanAutoCall = 1 << iota // allow auto property
 	clIdentAllowBuiltin
 	clIdentLHS
-	clIdentSelectorExpr // this ident is X of ast.SelectorExpr
+	clIdentSelectorExpr // this ident is X (not Sel) of ast.SelectorExpr
 	clIdentGoto
 	clCallWithTwoValue
 	clCommandWithoutArgs
-	clMayBuiltinNew // maybe is builtin directive `new`
+	clCommandIdent
 )
 
 const (
 	objNormal = iota
 	objPkgRef
 	objCPkgRef
+	objGopExec
 )
 
 const errorPkgPath = "github.com/qiniu/x/errors"
 
 func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg gox.PkgRef, kind int) {
 	fvalue := (flags&clIdentSelectorExpr) != 0 || (flags&clIdentLHS) == 0
+	cb := ctx.cb
 	name := ident.Name
 	if name == "_" {
 		if fvalue {
 			panic(ctx.newCodeError(ident.Pos(), "cannot use _ as value"))
 		}
-		ctx.cb.VarRef(nil)
+		cb.VarRef(nil)
 		return
 	}
 
+	var recv *types.Var
 	var oldo types.Object
 	scope := ctx.pkg.Types.Scope()
-	at, o := ctx.cb.Scope().LookupParent(name, token.NoPos)
+	at, o := cb.Scope().LookupParent(name, token.NoPos)
 	if o != nil {
 		if at != scope && at != types.Universe { // local object
 			goto find
@@ -96,19 +100,16 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg gox.PkgRef, k
 	}
 
 	if ctx.isClass { // in a Go+ class file
-		if fn := ctx.cb.Func(); fn != nil {
-			sig := fn.Ancestor().Type().(*types.Signature)
-			if recv := sig.Recv(); recv != nil {
-				ctx.cb.Val(recv)
-				chkFlag := flags // &^ clCommandWithoutArgs (TODO: why?)
-				if chkFlag&clIdentSelectorExpr != 0 {
-					chkFlag = clIdentCanAutoCall
-				}
-				if compileMember(ctx, ident, name, chkFlag) == nil { // class member object
-					return
-				}
-				ctx.cb.InternalStack().PopN(1)
+		if recv = classRecv(cb); recv != nil {
+			cb.Val(recv)
+			chkFlag := flags
+			if chkFlag&clIdentSelectorExpr != 0 { // TODO: remove this condition
+				chkFlag = clIdentCanAutoCall
 			}
+			if compileMember(ctx, ident, name, chkFlag) == nil { // class member object
+				return
+			}
+			cb.InternalStack().PopN(1)
 		}
 	}
 
@@ -151,6 +152,11 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg gox.PkgRef, k
 		}
 		oldo, o = o, obj
 	} else if o == nil {
+		// for support Gop_Exec, see TestSpxGopExec
+		if (clCommandIdent&flags) != 0 && recv != nil && gopExecVal(cb, recv, ident) == nil {
+			kind = objGopExec
+			return
+		}
 		if (clIdentGoto & flags) != 0 {
 			l := ident.Obj.Data.(*ast.Ident)
 			panic(ctx.newCodeErrorf(l.Pos(), "label %v is not defined", l.Name))
@@ -160,13 +166,13 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (pkg gox.PkgRef, k
 
 find:
 	if fvalue {
-		ctx.cb.Val(o, ident)
+		cb.Val(o, ident)
 	} else {
-		ctx.cb.VarRef(o, ident)
+		cb.VarRef(o, ident)
 	}
 	if rec := ctx.recorder(); rec != nil {
-		e := ctx.cb.Get(-1)
-		if oldo != nil && gox.IsTypeEx(e.Type) {
+		e := cb.Get(-1)
+		if oldo != nil && gox.IsTypeEx(e.Type) { // for builtin object
 			rec.recordIdent(ctx, ident, oldo)
 			return
 		}
@@ -175,6 +181,19 @@ find:
 	return
 }
 
+func classRecv(cb *gox.CodeBuilder) *types.Var {
+	if fn := cb.Func(); fn != nil {
+		sig := fn.Ancestor().Type().(*types.Signature)
+		return sig.Recv()
+	}
+	return nil
+}
+
+func gopExecVal(cb *gox.CodeBuilder, recv *types.Var, ident *ast.Ident) error {
+	_, e := cb.Val(recv).Member("Gop_Exec", gox.MemberFlagVal, ident)
+	return e
+}
+
 func isBuiltin(o types.Object) bool {
 	if _, ok := o.(*types.Builtin); ok {
 		return ok
@@ -187,8 +206,6 @@ func compileMember(ctx *blockCtx, v ast.Node, name string, flags int) error {
 	switch {
 	case (flags & clIdentLHS) != 0:
 		mflag = gox.MemberFlagRef
-	case (flags & clCommandWithoutArgs) != 0:
-		mflag = gox.MemberFlagMethodAlias
 	case (flags & clIdentCanAutoCall) != 0:
 		mflag = gox.MemberFlagAutoProperty
 	default:
@@ -220,14 +237,53 @@ func twoValue(inFlags []int) bool {
 	return inFlags != nil && (inFlags[0]&clCallWithTwoValue) != 0
 }
 
+func identOrSelectorFlags(inFlags []int) (flags int, cmdNoArgs bool) {
+	if inFlags == nil {
+		return clIdentCanAutoCall, false
+	}
+	flags = inFlags[0]
+	if cmdNoArgs = (flags & clCommandWithoutArgs) != 0; cmdNoArgs {
+		flags &^= clCommandWithoutArgs
+	} else {
+		flags |= clIdentCanAutoCall
+	}
+	return
+}
+
+func callCmdNoArgs(ctx *blockCtx, src ast.Node, panicErr bool) (err error) {
+	if gox.IsFunc(ctx.cb.InternalStack().Get(-1).Type) {
+		if err = ctx.cb.CallWithEx(0, 0, src); err != nil {
+			if panicErr {
+				panic(err)
+			}
+		}
+	}
+	return
+}
+
 func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 	switch v := expr.(type) {
 	case *ast.Ident:
-		flags := clIdentCanAutoCall
-		if inFlags != nil {
-			flags |= inFlags[0]
+		flags, cmdNoArgs := identOrSelectorFlags(inFlags)
+		if cmdNoArgs {
+			flags |= clCommandIdent // for support Gop_Exec, see TestSpxGopExec
+		}
+		_, kind := compileIdent(ctx, v, flags)
+		if cmdNoArgs {
+			cb := ctx.cb
+			if kind == objGopExec {
+				cb.Val(v.Name, v)
+			} else {
+				err := callCmdNoArgs(ctx, expr, false)
+				if err == nil {
+					return
+				}
+				if !(ctx.isClass && tryGopExec(cb, v)) {
+					panic(err)
+				}
+			}
+			cb.CallWith(1, 0, v)
 		}
-		compileIdent(ctx, v, flags)
 	case *ast.BasicLit:
 		compileBasicLit(ctx, v)
 	case *ast.CallExpr:
@@ -237,11 +293,12 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 		}
 		compileCallExpr(ctx, v, flags)
 	case *ast.SelectorExpr:
-		flags := clIdentCanAutoCall
-		if inFlags != nil {
-			flags |= inFlags[0]
-		}
+		flags, cmdNoArgs := identOrSelectorFlags(inFlags)
 		compileSelectorExpr(ctx, v, flags)
+		if cmdNoArgs {
+			callCmdNoArgs(ctx, expr, true)
+			return
+		}
 	case *ast.BinaryExpr:
 		compileBinaryExpr(ctx, v)
 	case *ast.UnaryExpr:
@@ -282,12 +339,8 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, inFlags ...int) {
 		compileErrWrapExpr(ctx, v, 0)
 	case *ast.FuncType:
 		ctx.cb.Typ(toFuncType(ctx, v, nil, nil), v)
-	case *ast.Ellipsis:
-		panic("compileEllipsis: ast.Ellipsis unexpected")
-	case *ast.KeyValueExpr:
-		panic("compileExpr: ast.KeyValueExpr unexpected")
 	default:
-		log.Panicln("compileExpr failed: unknown -", reflect.TypeOf(v))
+		log.Panicf("compileExpr failed: unknown - %T\n", v)
 	}
 	if rec := ctx.recorder(); rec != nil {
 		rec.recordExpr(ctx, expr, false)
@@ -472,9 +525,7 @@ func compilePkgRef(ctx *blockCtx, at gox.PkgRef, x *ast.Ident, flags, pkgKind in
 func identVal(ctx *blockCtx, x *ast.Ident, flags int, v types.Object, alias bool) bool {
 	autocall := false
 	if alias {
-		if (flags & clCommandWithoutArgs) != 0 {
-			autocall = true
-		} else if autocall = (flags & clIdentCanAutoCall) != 0; autocall {
+		if autocall = (flags & clIdentCanAutoCall) != 0; autocall {
 			if !gox.HasAutoProperty(v.Type()) {
 				return false
 			}
@@ -562,13 +613,20 @@ func (p *fnType) initFuncs(base int, funcs []types.Object) {
 }
 
 func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
-	var flags gox.InstrFlags
+	var ifn *ast.Ident
 	switch fn := v.Fun.(type) {
 	case *ast.Ident:
-		if len(v.Args) == 1 && fn.Name == "new" { // maybe new(T)
-			flags = clMayBuiltinNew
+		if v.IsCommand() { // for support Gop_Exec, see TestSpxGopExec
+			inFlags |= clCommandIdent
+		}
+		if _, kind := compileIdent(ctx, fn, clIdentAllowBuiltin|inFlags); kind == objGopExec {
+			args := make([]ast.Expr, 1, len(v.Args)+1)
+			args[0] = toBasicLit(fn)
+			args = append(args, v.Args...)
+			v = &ast.CallExpr{Fun: fn, Args: args, Ellipsis: v.Ellipsis, NoParenEnd: v.NoParenEnd}
+		} else {
+			ifn = fn
 		}
-		compileIdent(ctx, fn, clIdentAllowBuiltin|inFlags)
 	case *ast.SelectorExpr:
 		compileSelectorExpr(ctx, fn, 0)
 	case *ast.ErrWrapExpr:
@@ -584,10 +642,14 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
 	default:
 		compileExpr(ctx, fn)
 	}
-	var fnt = ctx.cb.Get(-1).Type
+	var err error
+	var stk = ctx.cb.InternalStack()
+	var base = stk.Len()
+	var fnt = stk.Get(-1).Type
+	var flags gox.InstrFlags
 	var ellipsis = v.Ellipsis != gotoken.NoPos
 	if ellipsis {
-		flags |= gox.InstrFlagEllipsis
+		flags = gox.InstrFlagEllipsis
 	}
 	if (inFlags & clCallWithTwoValue) != 0 {
 		flags |= gox.InstrFlagTwoValue
@@ -595,43 +657,80 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
 	fn := &fnType{}
 	fn.load(fnt)
 	for fn != nil {
-		err := compileCallArgs(fn, ctx, v, ellipsis, flags)
-		if err == nil {
-			break
-		}
-		if fn.next == nil {
-			panic(err)
+		if err = compileCallArgs(fn, ctx, v, ellipsis, flags); err == nil {
+			if rec := ctx.recorder(); rec != nil {
+				rec.recordCallExpr(ctx, v, fnt)
+			}
+			return
 		}
+		stk.SetLen(base)
 		fn = fn.next
 	}
-	if rec := ctx.recorder(); rec != nil {
-		rec.recordCallExpr(ctx, v, fnt)
+	if ifn != nil && builtinOrGopExec(ctx, ifn, v, flags) == nil {
+		return
 	}
+	panic(err)
+}
+
+func toBasicLit(fn *ast.Ident) *ast.BasicLit {
+	return &ast.BasicLit{ValuePos: fn.NamePos, Kind: token.STRING, Value: strconv.Quote(fn.Name)}
+}
+
+// maybe builtin new/delete: see TestSpxNewObj, TestMayBuiltinDelete
+// maybe Gop_Exec: see TestSpxGopExec
+func builtinOrGopExec(ctx *blockCtx, ifn *ast.Ident, v *ast.CallExpr, flags gox.InstrFlags) error {
+	cb := ctx.cb
+	switch name := ifn.Name; name {
+	case "new", "delete":
+		cb.InternalStack().PopN(1)
+		cb.Val(ctx.pkg.Builtin().Ref(name), ifn)
+		return fnCall(ctx, v, flags, 0)
+	default:
+		// for support Gop_Exec, see TestSpxGopExec
+		if v.IsCommand() && ctx.isClass && tryGopExec(cb, ifn) {
+			return fnCall(ctx, v, flags, 1)
+		}
+	}
+	return syscall.ENOENT
+}
+
+func tryGopExec(cb *gox.CodeBuilder, ifn *ast.Ident) bool {
+	if recv := classRecv(cb); recv != nil {
+		cb.InternalStack().PopN(1)
+		if gopExecVal(cb, recv, ifn) == nil {
+			cb.Val(ifn.Name, ifn)
+			return true
+		}
+	}
+	return false
+}
+
+func fnCall(ctx *blockCtx, v *ast.CallExpr, flags gox.InstrFlags, extra int) error {
+	for _, arg := range v.Args {
+		compileExpr(ctx, arg)
+	}
+	return ctx.cb.CallWithEx(len(v.Args)+extra, flags, v)
 }
 
 func compileCallArgs(fn *fnType, ctx *blockCtx, v *ast.CallExpr, ellipsis bool, flags gox.InstrFlags) (err error) {
-	cb := ctx.cb
-	stk := cb.InternalStack()
-	n := stk.Len()
-	defer func() {
-		if r := recover(); r != nil {
-			if e, ok := r.(error); ok {
-				err = e
-			} else {
-				src := ctx.LoadExpr(v)
-				err = ctx.newCodeErrorf(v.Pos(), "compile func %v error: %v", src, r)
-			}
-			stk.SetLen(n)
-		}
-	}()
 	for i, arg := range v.Args {
 		switch expr := arg.(type) {
 		case *ast.LambdaExpr:
-			sig := checkLambdaFuncType(ctx, expr, fn.arg(i, ellipsis), clLambaArgument, v.Fun)
-			compileLambdaExpr(ctx, expr, sig)
+			sig, e := checkLambdaFuncType(ctx, expr, fn.arg(i, ellipsis), clLambaArgument, v.Fun)
+			if e != nil {
+				return e
+			}
+			if err = compileLambdaExpr(ctx, expr, sig); err != nil {
+				return
+			}
 		case *ast.LambdaExpr2:
-			sig := checkLambdaFuncType(ctx, expr, fn.arg(i, ellipsis), clLambaArgument, v.Fun)
-			compileLambdaExpr2(ctx, expr, sig)
+			sig, e := checkLambdaFuncType(ctx, expr, fn.arg(i, ellipsis), clLambaArgument, v.Fun)
+			if e != nil {
+				return e
+			}
+			if err = compileLambdaExpr2(ctx, expr, sig); err != nil {
+				return
+			}
 		case *ast.CompositeLit:
 			compileCompositeLit(ctx, expr, fn.arg(i, ellipsis), true)
 		case *ast.SliceLit:
@@ -647,9 +746,11 @@ func compileCallArgs(fn *fnType, ctx *blockCtx, v *ast.CallExpr, ellipsis bool,
 			}
 			typetype := fn.typetype && t != nil
 			if typetype {
-				stk.Pop()
+				ctx.cb.InternalStack().PopN(1)
+			}
+			if err = compileSliceLit(ctx, expr, t, true); err != nil {
+				return
 			}
-			compileSliceLit(ctx, expr, t)
 			if typetype {
 				return
 			}
@@ -657,21 +758,7 @@ func compileCallArgs(fn *fnType, ctx *blockCtx, v *ast.CallExpr, ellipsis bool,
 			compileExpr(ctx, arg)
 		}
 	}
-	if (flags & clMayBuiltinNew) != 0 { // maybe new(T): see TestSpxNewObj
-		arg := stk.Get(-1)
-		if _, ok := arg.Type.(*gox.TypeType); ok && isNonInstr(stk.Get(-2).Type) {
-			stk.PopN(2)
-			cb.Val(ctx.pkg.Builtin().Ref("new"))
-			stk.Push(arg)
-		}
-	}
-	cb.CallWith(len(v.Args), flags, v)
-	return
-}
-
-func isNonInstr(t types.Type) bool {
-	_, ok := t.(*gox.TyInstruction)
-	return !ok
+	return ctx.cb.CallWithEx(len(v.Args), flags, v)
 }
 
 type clLambaFlag string
@@ -683,7 +770,7 @@ const (
 )
 
 // check lambda func type
-func checkLambdaFuncType(ctx *blockCtx, lambda ast.Expr, ftyp types.Type, flag clLambaFlag, toNode ast.Node) *types.Signature {
+func checkLambdaFuncType(ctx *blockCtx, lambda ast.Expr, ftyp types.Type, flag clLambaFlag, toNode ast.Node) (*types.Signature, error) {
 	typ := ftyp
 retry:
 	switch t := typ.(type) {
@@ -693,26 +780,29 @@ retry:
 				break
 			}
 		}
-		return t
+		return t, nil
 	case *types.Named:
 		typ = t.Underlying()
 		goto retry
 	}
 	src := ctx.LoadExpr(toNode)
-	err := ctx.newCodeErrorf(lambda.Pos(), "cannot use lambda literal as type %v in %v to %v", ftyp, flag, src)
-	panic(err)
+	return nil, ctx.newCodeErrorf(lambda.Pos(), "cannot use lambda literal as type %v in %v to %v", ftyp, flag, src)
 }
 
 func compileLambda(ctx *blockCtx, lambda ast.Expr, sig *types.Signature) {
 	switch expr := lambda.(type) {
 	case *ast.LambdaExpr:
-		compileLambdaExpr(ctx, expr, sig)
+		if err := compileLambdaExpr(ctx, expr, sig); err != nil {
+			panic(err)
+		}
 	case *ast.LambdaExpr2:
-		compileLambdaExpr2(ctx, expr, sig)
+		if err := compileLambdaExpr2(ctx, expr, sig); err != nil {
+			panic(err)
+		}
 	}
 }
 
-func makeLambdaParams(ctx *blockCtx, pos token.Pos, lhs []*ast.Ident, in *types.Tuple) *types.Tuple {
+func makeLambdaParams(ctx *blockCtx, pos token.Pos, lhs []*ast.Ident, in *types.Tuple) (*types.Tuple, error) {
 	pkg := ctx.pkg
 	n := len(lhs)
 	if nin := in.Len(); n != nin {
@@ -724,11 +814,11 @@ func makeLambdaParams(ctx *blockCtx, pos token.Pos, lhs []*ast.Ident, in *types.
 		for i, v := range lhs {
 			has[i] = v.Name
 		}
-		panic(ctx.newCodeErrorf(
-			pos, "too %s arguments in lambda expression\n\thave (%s)\n\twant %v", fewOrMany, strings.Join(has, ", "), in))
+		return nil, ctx.newCodeErrorf(
+			pos, "too %s arguments in lambda expression\n\thave (%s)\n\twant %v", fewOrMany, strings.Join(has, ", "), in)
 	}
 	if n == 0 {
-		return nil
+		return nil, nil
 	}
 	params := make([]*types.Var, n)
 	for i, name := range lhs {
@@ -738,7 +828,7 @@ func makeLambdaParams(ctx *blockCtx, pos token.Pos, lhs []*ast.Ident, in *types.
 			rec.Def(name, param)
 		}
 	}
-	return types.NewTuple(params...)
+	return types.NewTuple(params...), nil
 }
 
 func makeLambdaResults(pkg *gox.Package, out *types.Tuple) *types.Tuple {
@@ -753,9 +843,12 @@ func makeLambdaResults(pkg *gox.Package, out *types.Tuple) *types.Tuple {
 	return types.NewTuple(results...)
 }
 
-func compileLambdaExpr(ctx *blockCtx, v *ast.LambdaExpr, sig *types.Signature) {
+func compileLambdaExpr(ctx *blockCtx, v *ast.LambdaExpr, sig *types.Signature) error {
 	pkg := ctx.pkg
-	params := makeLambdaParams(ctx, v.Pos(), v.Lhs, sig.Params())
+	params, err := makeLambdaParams(ctx, v.Pos(), v.Lhs, sig.Params())
+	if err != nil {
+		return err
+	}
 	results := makeLambdaResults(pkg, sig.Results())
 	ctx.cb.NewClosure(params, results, false).BodyStart(pkg)
 	if len(v.Lhs) > 0 {
@@ -765,11 +858,15 @@ func compileLambdaExpr(ctx *blockCtx, v *ast.LambdaExpr, sig *types.Signature) {
 		compileExpr(ctx, v)
 	}
 	ctx.cb.Return(len(v.Rhs)).End(v)
+	return nil
 }
 
-func compileLambdaExpr2(ctx *blockCtx, v *ast.LambdaExpr2, sig *types.Signature) {
+func compileLambdaExpr2(ctx *blockCtx, v *ast.LambdaExpr2, sig *types.Signature) error {
 	pkg := ctx.pkg
-	params := makeLambdaParams(ctx, v.Pos(), v.Lhs, sig.Params())
+	params, err := makeLambdaParams(ctx, v.Pos(), v.Lhs, sig.Params())
+	if err != nil {
+		return err
+	}
 	results := makeLambdaResults(pkg, sig.Results())
 	comments, once := ctx.cb.BackupComments()
 	fn := ctx.cb.NewClosure(params, results, false)
@@ -780,6 +877,7 @@ func compileLambdaExpr2(ctx *blockCtx, v *ast.LambdaExpr2, sig *types.Signature)
 	compileStmts(ctx, v.Body.List)
 	cb.End(v)
 	ctx.cb.SetComments(comments, once)
+	return nil
 }
 
 func compileFuncLit(ctx *blockCtx, v *ast.FuncLit) {
@@ -920,7 +1018,10 @@ func compileStructLitInKeyVal(ctx *blockCtx, elts []ast.Expr, t *types.Struct, t
 		}
 		switch expr := kv.Value.(type) {
 		case *ast.LambdaExpr, *ast.LambdaExpr2:
-			sig := checkLambdaFuncType(ctx, expr, t.Field(idx).Type(), clLambaField, kv.Key)
+			sig, err := checkLambdaFuncType(ctx, expr, t.Field(idx).Type(), clLambaField, kv.Key)
+			if err != nil {
+				panic(err)
+			}
 			compileLambda(ctx, expr, sig)
 		default:
 			compileExpr(ctx, kv.Value)
@@ -1039,7 +1140,14 @@ func compileCompositeLit(ctx *blockCtx, v *ast.CompositeLit, expected types.Type
 	}
 }
 
-func compileSliceLit(ctx *blockCtx, v *ast.SliceLit, typ types.Type) {
+func compileSliceLit(ctx *blockCtx, v *ast.SliceLit, typ types.Type, noPanic ...bool) (err error) {
+	if noPanic != nil {
+		defer func() {
+			if e := recover(); e != nil { // TODO: don't use defer to capture error
+				err = ctx.recoverErr(e, v)
+			}
+		}()
+	}
 	n := len(v.Elts)
 	for _, elt := range v.Elts {
 		compileExpr(ctx, elt)
@@ -1049,6 +1157,7 @@ func compileSliceLit(ctx *blockCtx, v *ast.SliceLit, typ types.Type) {
 	} else {
 		ctx.cb.SliceLitEx(typ, n, false, v)
 	}
+	return
 }
 
 func compileRangeExpr(ctx *blockCtx, v *ast.RangeExpr) {
