@@ -29,6 +29,15 @@ type MyGame struct {
 func Gopt_MyGame_Main(game interface{}) {
 }
 
+func (p *MyGame) Ls(n int) {}
+
+func (p *MyGame) Capout(doSth func()) (string, error) {
+	return "", nil
+}
+
+func (p *MyGame) Gop_Exec(name string, args ...any) {
+}
+
 func (p *MyGame) InitGameApp(args ...string) {
 }
 
