@@ -80,10 +80,13 @@ func runCmd(cmd *base.Command, args []string) {
 	}
 
 	noChdir := *flagNoChdir
-	conf, err := gop.NewDefaultConf(".")
+	conf, err := gop.NewDefaultConf(".", true)
 	if err != nil {
 		log.Panicln("gop.NewDefaultConf:", err)
 	}
+	if !conf.Mod.HasModfile() { // if no go.mod, check GopDeps
+		conf.GopDeps = new(int)
+	}
 	confCmd := &gocmd.Config{Gop: conf.Gop}
 	confCmd.Flags = pass.Args
 	run(proj, args, !noChdir, conf, confCmd)
